/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe;

import org.apache.commons.lang.StringUtils;

import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Version;
import de.willuhn.jameica.system.Application;

/**
 * Settings f&uuml;r Mahoe
 *
 * @author Frank Mnster
 */
public final class Settings {

    /**
     * Settings Objekt of Jameica
     */
    private static final de.willuhn.jameica.system.Settings MAHOE_SETTINGS = Application.getPluginLoader()
            .getPlugin(MahoeMain.class).getResources().getSettings();

    /**
     * old version in case of an update
     */
    private static Version oldVersion;

    /**
     * Mahoe Plugin
     */
    private static final AbstractPlugin PLUGIN = Application.getPluginLoader().getPlugin(MahoeMain.class);

    /**
     * Selenium Plugin installed or not
     */
    private static boolean seleniumInstalled;

    /**
     * @return the oldVersion
     */
    public static Version getOldVersion() {

        return Settings.oldVersion;
    }

    /**
     * Returns the Selenium FF driver to use
     *
     * @return the Selenium FF driver
     */
    public static String getSeleniumDriver() {

        return Settings.MAHOE_SETTINGS.getString("Mahoe.seleniumDriver", StringUtils.EMPTY); //$NON-NLS-1$
    }

    /**
     * Returns the Selenium FF binary to use
     *
     * @return the Selenium FF binary
     */
    public static String getSeleniumFireFoxBinary() {

        return Settings.MAHOE_SETTINGS.getString("Mahoe.seleniumFireFoxBinary", StringUtils.EMPTY); //$NON-NLS-1$
    }

    /**
     * Liefert die Mahoe-Version
     *
     * @return die Mahoe-Version
     */
    public static String getVersion() {

        return Settings.PLUGIN.getManifest().getVersion().toString(); // NOPMD
                                                                      // on
                                                                      // 20.06.21,
                                                                      // 19:28
    }

    /**
     * Return if the back end class update run already.
     *
     * @return <i>true</i> if the back end class update had run successfully,
     *         <i>false</i> else.
     */
    public static boolean isBackEndClassUpdated() {

        return Settings.MAHOE_SETTINGS.getBoolean("Mahoe.backEndClassUpdated", false); //$NON-NLS-1$
    }

    /**
     * Returns if the Selenium FF session should be closed automatically after
     * each job group (bank and account are the same) or to keep the session
     * open and the user needs to close it manually.
     */
    public static boolean isSeleniumAutoClose() {

        return Settings.MAHOE_SETTINGS.getBoolean("Mahoe.seleniumAutoClose", false); //$NON-NLS-1$
    }

    /**
     * Is BOS function used
     *
     * @return isBosUsed
     */
    public static boolean isSeleniumInstalled() {

        return Settings.seleniumInstalled;
    }

    /**
     * Returns if the Selenium log level should be set to DEBUG
     */
    public static boolean isSeleniumUseDegugLog() {

        return Settings.MAHOE_SETTINGS.getBoolean("Mahoe.seleniumDebugLog", false); //$NON-NLS-1$
    }

    /**
     * Set if the back end class update run already.
     *
     * @param backEndClassUpdated
     *            set <i>true</i> if the back end class update run successfully
     *            already, <i>false</i> else.
     */
    public static void setBackEndClassUpdated(final Boolean backEndClassUpdated) {

        Settings.MAHOE_SETTINGS.setAttribute("Mahoe.backEndClassUpdated", backEndClassUpdated); //$NON-NLS-1$
    }

    /**
     * Set the FF profile to use
     *
     * @param ffProfile
     *            the FF profile driver to use
     */
    public static void setFfProfile(final String ffProfile) {

        Settings.MAHOE_SETTINGS.setAttribute("Mahoe.ffProfile", ffProfile); //$NON-NLS-1$
    }

    /**
     * @param oldVersion
     *            the oldVersion to set
     */
    public static void setOldVersion(final Version oldVersion) {

        Settings.oldVersion = oldVersion;
    }

    /**
     * Set if the Selenium FF session should be closed automatically after each
     * job group (bank and account are the same) or to keep the session open and
     * the user needs to close it manually.
     *
     * @param autoClose
     *            <code>true</code> if the window should be closed after each
     *            job group, <code>false</code> if the user should close the
     *            window automatically.
     */
    public static void setSeleniumAutoClose(final boolean autoClose) {

        Settings.MAHOE_SETTINGS.setAttribute("Mahoe.seleniumAutoClose", autoClose); //$NON-NLS-1$
    }

    /**
     * Set the Selenium FF driver to use
     *
     * @param seleniumDriver
     *            the Selenium FF driver to use
     */
    public static void setSeleniumDriver(final String seleniumDriver) {

        Settings.MAHOE_SETTINGS.setAttribute("Mahoe.seleniumDriver", seleniumDriver); //$NON-NLS-1$
    }

    /**
     * Set the Selenium FF binary to use
     *
     * @param seleniumFireFoxBinary
     *            the Selenium FF binary to use
     */
    public static void setSeleniumFireFoxBinary(final String seleniumFireFoxBinary) {

        Settings.MAHOE_SETTINGS.setAttribute("Mahoe.seleniumFireFoxBinary", seleniumFireFoxBinary); //$NON-NLS-1$
    }

    /**
     * Set if the Selenium Ff driver is installed
     *
     * @param seleniumInstalled
     */
    public static void setSeleniumInstalled(final Boolean seleniumInstalled) {

        Settings.seleniumInstalled = seleniumInstalled;
    }

    /**
     * Set the Selenium log level
     *
     * @param seleniumDebugLog
     *            set to <i>true</i> in case debug log level should be used
     */
    public static void setSeleniumUseDebugLog(final boolean seleniumDebugLog) {

        Settings.MAHOE_SETTINGS.setAttribute("Mahoe.seleniumDebugLog", seleniumDebugLog); //$NON-NLS-1$
    }

    /**
     * Standardkonstruktor
     */
    private Settings() {

        // Absichtlich leer
    }
}
