/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.aktien.jobs;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.htmlunit.WebClient;

import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.boerseard.ArdStockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.BoerseArdApi;
import de.frankmuenster.mahoe.banks.aktien.Aktien;
import de.frankmuenster.mahoe.banks.aktien.jobs.web.AktienBankMessages;
import de.frankmuenster.mahoe.banks.aktien.jobs.web.AktienWebAccountInformation;
import de.frankmuenster.mahoe.banks.aktien.jobs.web.AktienWebSite;
import de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobKontoauszug;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.TransactionSaver;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.hbci.messaging.ObjectChangedMessage;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Der Job der die eigentliche Arbeit macht.
 *
 * @author Frank Mnster
 */
public class AktienSynchronizeJobKontoauszug
        extends AbstractMahoeSynchronizeJobKontoauszug<WebClient, AktienWebAccountInformation, AktienBankMessages> {

    /**
     * Standartkonstruktor
     */
    public AktienSynchronizeJobKontoauszug() {

        super();
    }

    /**
     * @see AbstractMahoeSynchronizeJobKontoauszug#getWebSite()
     */
    @Override
    protected void getWebSite() throws WebSiteException, ApplicationException {

        this.webSite = AktienWebSite.getInstance(this.konto);
    }

    /**
     * @see AbstractMahoeSynchronizeJobKontoauszug#handleTransactions()
     */
    @Override
    protected void handleTransactions() throws WebSiteException, ApplicationException {

        // Mssen wir ganz anders machen
        int updatedStocks = 0;
        BigDecimal saldo = BigDecimal.ZERO;
        this.monitor.setPercentComplete(0);
        try {
            this.monitor.setStatusText(I18N.tr("AktienSynchronizeJobKontoauszug.updateStockValues")); //$NON-NLS-1$
            @SuppressWarnings("unchecked")
            final DBIterator<Umsatz> stocks = this.konto.getUmsaetze();
            // save total number of stocks
            updatedStocks = stocks.size();
            // calculate percentage step size
            int prozendStep = 100 / stocks.size();
            // List to hold not updated stocks
            final ArrayList<String> updateErrors = new ArrayList<>();
            // Loop over stocks
            while (stocks.hasNext()) {
                this.monitor.setPercentComplete(prozendStep++);
                final Umsatz stock = stocks.next();
                // Update on stock and return the saldo if successful
                final BigDecimal stockSaldo = this.updateStock(stock, updateErrors);
                if (stockSaldo != null) {
                    saldo = saldo.add(stockSaldo);
                }
            }
            if (!updateErrors.isEmpty()) {
                final StringBuilder errors = new StringBuilder(120)
                        .append(I18N.tr("AktienSynchronizeJobKontoauszug.listOfFailedStocks")); //$NON-NLS-1$
                for (final String stockName : updateErrors) {
                    errors.append("- ").append(stockName).append('\n'); //$NON-NLS-1$
                }
                Utils.notifyUsers(errors.toString());
            }
            // Calculate updated stocks
            updatedStocks -= updateErrors.size();
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("AktienSynchronizeJobKontoauszug.ErrorReadingDB") + e.getLocalizedMessage(), e); //$NON-NLS-1$
            throw new ApplicationException(e);
        }
        this.monitor.setStatusText(
                I18N.tr("AktienSynchronizeJobKontoauszug.savedUpdated", Integer.toString(updatedStocks))); //$NON-NLS-1$
        Logger.info(I18N.tr("AktienSynchronizeJobKontoauszug.savedUpdated", Integer.toString(updatedStocks))); //$NON-NLS-1$
        // Saldo/verfgbarer Betrag setzten
        TransactionSaver.saveSaldo(this.konto, saldo);
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.newSaldoIs", //$NON-NLS-1$
                saldo.toPlainString()));
    }

    /**
     * @see AbstractMahoeSynchronizeJobKontoauszug#login()
     */
    @Override
    protected void login() throws WebSiteException, ApplicationException {

        // Brauchen wir hier nicht
    }

    /**
     * Updates a stock with the actual values and store it in the DB.
     *
     * @param stock
     *            the stock to update
     * @param updateErrors
     *            the list of stock to add this stock in case of an error.
     * @return the saldo of the stock or <code>NULL</code> in case of an error
     */
    private BigDecimal updateStock(final Umsatz stock, final List<String> updateErrors) {

        BigDecimal saldo = null;

        final ArdStockInfoBean ardStock = new ArdStockInfoBean();
        String stockName = StringUtils.EMPTY;

        try {
            stockName = stock.getGegenkontoName() + ", ID: " //$NON-NLS-1$
                    + stock.getMeta(Aktien.STOCK_ID, I18N.tr("AktienSynchronizeJobKontoauszug.missingID")); //$NON-NLS-1$

            // set all stock info
            ardStock.setStockExchangeId(stock.getMeta(Aktien.STOCK_ID, StringUtils.EMPTY));
            ardStock.setName(stock.getGegenkontoName());
            ardStock.setIsin(stock.getMeta(Aktien.ISIN, StringUtils.EMPTY));
            ardStock.setWkn(stock.getMeta(Aktien.WKN, StringUtils.EMPTY));
            ardStock.setMainPageUrlName(stock.getMeta(Aktien.MAIN_URL, StringUtils.EMPTY));

            // Update values
            BoerseArdApi.getInstance().getActualValue(ardStock);

            // save new values
            stock.setValuta(ardStock.getDate());
            stock.setDatum(ardStock.getDate());
            stock.setBetrag(ardStock.getActValue().doubleValue());
            // Anzahl
            String amountStr = stock.getZweck();
            amountStr = amountStr.substring(amountStr.indexOf('=') + 1);
            amountStr = StringUtils.replace(amountStr, ".", StringUtils.EMPTY); //$NON-NLS-1$
            amountStr = StringUtils.replace(amountStr, ",", "."); //$NON-NLS-1$//$NON-NLS-2$
            final BigDecimal amount = new BigDecimal(amountStr);
            final BigDecimal total = amount.multiply(ardStock.getActValue());
            stock.setSaldo(total.doubleValue());
            stock.store();
            if (StringUtils.isEmpty(stock.getMeta(Aktien.STOCK_EXCHANGE, StringUtils.EMPTY))) {
                stock.setMeta(Aktien.STOCK_EXCHANGE, ardStock.getExchange());
            }
            stock.setKommentar(I18N.tr("AktienSynchronizeJobKontoauszug.exchange") //$NON-NLS-1$
                    + stock.getMeta(Aktien.STOCK_EXCHANGE, StringUtils.EMPTY));
            Application.getMessagingFactory().sendMessage(new ObjectChangedMessage(stock));

            saldo = BigDecimal.valueOf(stock.getSaldo()).setScale(2, RoundingMode.HALF_UP);

        } catch (RemoteException | FinanceException | ApplicationException e) {

            if (StringUtils.isNotEmpty(stockName)) {
                updateErrors.add(stockName);
            }
            Logger.error(I18N.tr("AktienSynchronizeJobKontoauszug.stockUpdateFailed") + e.getLocalizedMessage(), e); //$NON-NLS-1$
        }

        return saldo;
    }
}
