/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bsh.jobs;

import java.rmi.RemoteException;
import java.util.Arrays;
import java.util.List;

import org.htmlunit.WebClient;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.banks.bsh.SchwaebischHall;
import de.frankmuenster.mahoe.banks.bsh.web.BshBankMessages;
import de.frankmuenster.mahoe.banks.bsh.web.BshWebAccountInformation;
import de.frankmuenster.mahoe.banks.bsh.web.BshWebSite;
import de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobKontoauszug;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.PasswdCache;
import de.frankmuenster.mahoe.utils.TransactionSaver;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Hier wird die eigentliche Arbeit gemacht.
 *
 * @author Frank Mnster
 */
public class BshSynchronizeJobKontoauszug
        extends AbstractMahoeSynchronizeJobKontoauszug<WebClient, BshWebAccountInformation, BshBankMessages> {

    /**
     * Standartkonstruktor
     */
    public BshSynchronizeJobKontoauszug() {
        super();
    }

    /**
     * @see de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobKontoauszug#getWebSite()
     */
    @Override
    protected void getWebSite() throws WebSiteException, ApplicationException {

        this.webSite = BshWebSite.getInstance(this.konto);
    }

    /**
     * @see AbstractMahoeSynchronizeJobKontoauszug#handleTransactions()
     */
    @Override
    protected void handleTransactions() throws WebSiteException, ApplicationException {

        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.getBooked", //$NON-NLS-1$
                this.accountInfo.getLastTransactionDateAsString()));
        final List<Umsatz> booked = this.webSite.getBookedTransactions();

        // Debug-Ausgabe
        if (Logger.isLogging(Level.DEBUG)) {
            for (final Umsatz umsatz : booked) {

                String umsatzString;
                try {
                    umsatzString = I18N.tr("SynchronizeJobKontoauszug.transactionInfoLog", //$NON-NLS-1$
                            umsatz.getDatum().toString(), umsatz.getZweck(), umsatz.getZweck2(),
                            Double.toString(umsatz.getBetrag()), Double.toString(umsatz.getSaldo()),
                            umsatz.getKommentar());
                    Logger.debug(umsatzString);
                } catch (final RemoteException e) {
                    Logger.error(I18N.tr("SynchronizeJobKontoauszug.createTransactionInfoStringFailed"), e); //$NON-NLS-1$
                }
            }
        }

        // Speicher gebuchte Umstze
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savingBooked")); //$NON-NLS-1$
        final int bookedCount = TransactionSaver.saveBooked(this.konto, booked);
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$
        this.monitor.setPercentComplete(50);

        Logger.info(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$

    }

    /**
     * @see AbstractMahoeSynchronizeJobKontoauszug#login()
     */
    @Override
    protected void login() throws WebSiteException, ApplicationException {

        char[] password = null;
        // Passwortabfrage falls gewnscht
        try {
            if ("JA".equals(this.konto.getMeta(MahoeSynchronizeBackend.PW_ASK_ALWAYS, "NEIN"))) { //$NON-NLS-1$ //$NON-NLS-2$
                final String pWord = Application.getCallback()
                        .askPassword(I18N.tr("PasswdCache.requestPw", this.kundennummer, SchwaebischHall.NAME)); //$NON-NLS-1$
                password = pWord.toCharArray();
                PasswdCache.revoke(SchwaebischHall.NAME, this.kundennummer);
            } else {
                password = PasswdCache.getPassword(SchwaebischHall.NAME, this.kundennummer);
            }

            this.webSite.login(password);
        } catch (final WebSiteException e) {
            if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)) {
                // Passwort zurcksetzen
                PasswdCache.revoke(SchwaebischHall.NAME, this.kundennummer);
            }
            throw e;

        } catch (final Exception e) { // NOPMD
            final String msg = I18N.tr("BshSynchronizeJobKontoauszug.errorGettingPassword"); //$NON-NLS-1$
            Logger.error(msg, e);
            throw new ApplicationException(msg, e);
        } finally {
            if (password != null) {
                Arrays.fill(password, ' ');
            }
        }
    }

}
