/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bsh.web;

import java.util.Date;
import java.util.List;

import org.htmlunit.WebClient;

import de.frankmuenster.mahoe.messages.AbstractBankMessages;
import de.frankmuenster.mahoe.messages.MahoeMessage;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Bank-Nachrichten der BSH
 *
 * @author Frank Mnster
 */
public class BshBankMessages extends AbstractBankMessages<WebClient, BshWebAccountInformation> {

    /**
     * Konstruktor
     *
     * @param webClient
     * @param accountInfo
     *
     */
    public BshBankMessages(final WebClient webClient, final BshWebAccountInformation accountInfo) {
        super(webClient, accountInfo);
    }

    /**
     * @see de.frankmuenster.mahoe.messages.InterfaceBankMessages#markAsRead(de.frankmuenster.mahoe.messages.MahoeMessage)
     */
    @Override
    public void markAsRead(final MahoeMessage message) throws ApplicationException {

        try {
            BshWebSiteUtils.markMessageAsRead(this.webclient, message);
        } catch (final WebSiteException e) {
            Logger.error(I18N.tr("BshBankMessages.markAsReadFailed"), e); //$NON-NLS-1$
            throw new ApplicationException(I18N.tr("BshBankMessages.markAsReadFailed"), e); //$NON-NLS-1$
        }

    }

    /**
     * @see AbstractBankMessages#readMessages(Date)
     */
    @Override
    public void readMessages(final Date startDate) throws WebSiteException, ApplicationException {

        if (BshWebSiteUtils.getNewMessageCount(this.webclient) > 0) {
            Logger.info("Lese neue Institutsnachrichten."); //$NON-NLS-1$

            final List<MahoeMessage> mesgs = BshWebSiteUtils.getNewMessages(this.webclient);
            this.messages.addAll(mesgs);

        }

    }
}
