
package de.frankmuenster.mahoe.messages;

import java.util.Date;

import de.frankmuenster.mahoe.web.WebSiteException;
import de.willuhn.util.ApplicationException;

/**
 * Interface fr die Abfrage von Bank-Nachrichten
 *
 * @author Frank Mnster
 */
public interface InterfaceBankMessages {

    /**
     * Markiert eine Nachricht als gelesen.
     *
     * @param message
     *            Die Nachricht, die als gelesen markiert werden soll
     * @throws ApplicationException
     */
    void markAsRead(MahoeMessage message) throws ApplicationException;

    /**
     * Liest alle Bank-Nachrichten ab startDate. Ist startDate nicht gegeben,
     * wird das letzte Jahr verwendet. Muss von der realen Klasse implementiert
     * werden.
     *
     * @param startDate
     * @throws WebSiteException
     * @throws ApplicationException
     */
    void readMessages(Date startDate) throws WebSiteException, ApplicationException;

    /**
     * Speichert eine Bank-Nachricht. Allerdings nur wenn sie nicht schon
     * vorhanden ist.
     *
     * @return Anzahl an neuen Nachrichten
     * @throws ApplicationException
     */
    int saveMessage() throws ApplicationException;

}
