/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.utils;

import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.io.IOUtils;
import org.apache.http.entity.ContentType;
import org.htmlunit.WebResponse;
import org.htmlunit.html.HtmlAnchor;

import de.willuhn.logging.Logger;

/**
 * Utility class for web methods
 *
 * @author Frank Mnster
 */
public final class WebUtilsHtmlUnit {

    /**
     * Read the account statement PDF from an HTMLUnit anchor into a byte array.
     *
     * @param link
     *            the HMTLUnit anchor to the PDF
     * @return the PDF as byte array
     */
    public static byte[] getPdfAsByteArray(final HtmlAnchor link) {

        byte[] data = null;
        try (InputStream inputStream = link.click().getWebResponse().getContentAsStream()) {

            data = IOUtils.toByteArray(inputStream);

        } catch (final IOException exception) {
            Logger.error(exception.getMessage());
        }

        return data;

    }

    /**
     * berprft die Webantwort, ob der http Statuscode 200 ist und die Antwort
     * im JSON-Format vorliegt.
     *
     * @param webResponse
     * @return true wenn HHTP code 200 und Antwortinhalt application/json ist,
     *         sonst false.
     */
    public static boolean webResponseIsJson(final WebResponse webResponse) { // NOPMD

        boolean answerOK = false;
        if (webResponse.getStatusCode() == org.apache.http.HttpStatus.SC_OK
                && ContentType.APPLICATION_JSON.getMimeType().equalsIgnoreCase(webResponse.getContentType())) { // $NON-NLS-1$
            answerOK = true;

        } else {
            // Ein Debuginfos loggen
            Logger.debug("JSON-Check-Status: " + webResponse.getStatusCode()); //$NON-NLS-1$
            Logger.debug("JSON-Check-MimeType: " + webResponse.getContentType()); //$NON-NLS-1$
            Logger.debug("JSON-Check-Inhalt: " + webResponse.getContentAsString()); //$NON-NLS-1$
        }
        return answerOK;
    }

    /**
     * private class constructor
     */
    private WebUtilsHtmlUnit() {

        // intentionally left blank
    }

}
